RuntimeCheck.Check (runtime_check v0.1.0)
View SourceRepresents a system check.
- name: The name of the check, an atom or string
- checker: The function executed for the check, can be nil. It's as if the function returned
:ok - nested_checks: A list of
Checkthat are verified only if thecheckerreturned:ok
Summary
Functions
Runs the given check(s). Logs the result of each check if log is true.